/*********/
/* cpu.v */
/*********/

/*
Top module || Sub modules
============================
     cpu.v + rom8x1024_sim.v, ߥ졼 ROM
           |
           + rom8x1024_DE2.v, ư¸ ROM
           |
           + ram8x2048_sim.v, ߥ졼 RAM
           |
           + ram8x2048_DE2.v, ư¸ RAM
           |
           + alu.v, 黻˥å(ALU)
           |
           + pc.v, ץ५(PC)
           |
           + registers.v, 쥸ե
           |
           + plus4.v, PC 4 󥯥
           |
           + adder32.v, 32-bit ûϩ
           |
           + mux32_32_32.v, 32-bit 2  1 ϩ
           |
           + mux5_5_5.v, 5-bit 2  1 ϩ
           |
           + main_ctrl.v, ᥤϩ
           |
           + alu_ctrler.v, ALU ϩ
           |
           + shifter32_32_l2.v, 32-bit  32-bit  2-bit եȲϩ
           |
           + signext16_32.v, 16-bit  32-bit  ĥϩ
           |
           - is_branch.v, ʬϩ
============================
*/

//                   +----+
//              clk->|    |->o_clock
//            reset->|    |->o_pc[11:0]
//                   |    |->o_alu_y[31:0]
//                   |    |->o_alu_a[31:0]
//                   |    |->o_alu_b[31:0]
//                   |    |->o_alu_ctrl[3:0]
//                   |    |->o_alu_comp[1:0]
//        clk_ctrl0->|    |->o_reg_data1[31:0]
//        clk_ctrl1->|    |->o_reg_idx1[4:0]
//    clk_ctrl1_clk->|    |->o_reg_data2[31:0]
//                   |    |->o_reg_idx2[4:0]
//                   |    |->o_reg_wdata[31:0]
//                   |    |->o_reg_widx[4:0]
//                   |    |->o_reg_wen
//                   |    |->o_ram_addr
//       i_ram_addr->|    |->o_ram_rdata
//      i_ram_wdata->|    |->o_ram_wdata
//        i_ram_wen->|    |->o_ram_wen
//                   +----+

//
//ߥ졼
//DE2, quartus, ư¸ȥ
//`include "rom8x1024_sim.v"  // ROMεҡߥ졼ѡ
//`include "ram8x2048_sim.v"  // RAMεҡߥ졼ѡ

//
//ư¸
//ߥ졼ȥ
`include "rom8x1024_DE2.v"  // ROMεҡDE2ܡ, quartus, ư¸ѡ
`include "ram8x2048_DE2.v"  // RAMεҡDE2ܡ, quartus, ư¸ѡ

`include "alu.v"           // ALUε
`include "pc.v"            // PCε
`include "registers.v"     // 쥸եε
`include "plus4.v"         // PC󥯥Ȳϩε
`include "adder32.v"       // PCûϩε
`include "mux32_32_32.v"   // 32-bit, 32-bit,
                           // 32-bit(alu_b,reg_sel)ϩε
`include "mux5_5_5.v"      // 5-bit, 5-bit,
                           // 5-bit(reg_widx_sel1, reg_widx_sel2)ϩε
`include "main_ctrl.v"     // ᥤϩε
`include "alu_ctrler.v"    // ALU ϩε
`include "shifter32_32_l2.v"   // 32-bit, 32-bitϤ2ӥåȺեȲϩε
`include "signext16_32.v"      // 16-bit, 32-bitϤĥϩε
`include "is_branch.v"         // ʬϩε

`define max_bunsyuu_reg_val_2Hz 12587500    //  2Hz
`define max_bunsyuu_reg_val_10Hz   2517500  //  10Hz, 2517500 = 12587500/5
`define max_bunsyuu_reg_val_20Hz   1258750  //  20Hz, 1258750 = 12587500/10
`define max_bunsyuu_reg_val_100Hz   251750  //  100Hz,  251750 = 12587500/50
`define max_bunsyuu_reg_val_200Hz   125875  //  200Hz,  125875 = 12587500/100
`define max_bunsyuu_reg_val_1kHz    25175   //   1kHz,   25175 = 12587500/500
`define max_bunsyuu_reg_val_10kHz    2517   //  10kHz,    2517 = 12587500/5000

module cpu (clk, reset,
            clk_ctrl0, clk_ctrl1, clk_ctrl_clk,
            o_clock, o_pc, o_alu_y, o_alu_a, o_alu_b, o_alu_ctrl, o_alu_comp,
            o_reg_data1, o_reg_idx1, o_reg_data2, o_reg_idx2, o_reg_wdata, o_reg_widx, o_reg_wen,
            o_ram_addr, o_ram_rdata, o_ram_wdata, o_ram_wen,
            i_ram_addr, i_ram_wdata, i_ram_wen
);

  // Input ports
  input clk;           // å(25.175 MHz)
  input reset;         // ꥻåȿ

  input clk_ctrl0;     //
  input clk_ctrl1;     // (clk_ctrl1, clk_ctrl0)=(0,0)2Hz, (0,1)20Hz, (1,0)200Hz, (1,1)겡å
  input clk_ctrl_clk;  //겡å

  input [12:0]  i_ram_addr;    // RAM address
  input [31:0] i_ram_wdata;    // RAM write data
  input          i_ram_wen;    // RAM write enable

  // Output ports
  output            o_clock;
  output [11:0]        o_pc;    // PC
  output [31:0]     o_alu_y;    // ALU result
  output [31:0]     o_alu_a;    // ALU in a
  output [31:0]     o_alu_b;    // ALU in b
  output [3:0]   o_alu_ctrl;    // ALU ctrl
  output [1:0]   o_alu_comp;    // ALU comp
  output [31:0] o_reg_data1;    // 쥸 out data1
  output [4:0]   o_reg_idx1;    // 쥸 read idx1
  output [31:0] o_reg_data2;    // 쥸 out data2
  output [4:0]   o_reg_idx2;    // 쥸 read idx2
  output [31:0] o_reg_wdata;    // 쥸 write data
  output [4:0]   o_reg_widx;    // 쥸 write idx
  output          o_reg_wen;    // 쥸 write enable
  output [12:0]  o_ram_addr;    // RAM address
  output [31:0] o_ram_rdata;    // RAM read data
  output [31:0] o_ram_wdata;    // RAM write data
  output          o_ram_wen;    // RAM write enable

  //
  //
  // ʬ
  // Registers(ʬ)
  wire [23:0] max_bunsyuu_reg_val;

  reg            clock_reg;  //  1-bit 쥸
  reg  [23:0]  bunsyuu_reg;  // 24-bit 쥸

  // Wire(ʬ)
  wire               clock;      //  1-bit wire
  wire           clock_rom;      //  1-bit wire
  wire           clock_ram;      //  1-bit wire

  // Assignment(ʬ)
  // (clk_ctrl1, clk_ctrl0)=(0,0)2Hz, (0,1)200Hz, (1,0)1kHz, (1,1)겡å
  assign clock         = ((clk_ctrl1 == 1'b1) && (clk_ctrl0 == 1'b1)) ? clk_ctrl_clk : clock_reg;  // wire
  assign clock_rom     =   bunsyuu_reg[3];  // wire
  assign clock_ram     =   bunsyuu_reg[4];  // wire

  // Always֥å: åʬ
  // : clk, reset
  // : clock_reg
  // ǽ: ꥸʥclk(50MHz) 
  //       ʬclock(2Hz, 200Hz, 1kHz, ʬʤ겡å),
  //                (clk_ctrl1, clk_ctrl0)=(0,0)2Hz, (0,1)200Hz, (1,0)1kHz, (1,1)겡å
  //       clock μȿϥƬ define ʸǷ.
  // 쥸: bunsyuu_reg, clk_reg
  assign max_bunsyuu_reg_val = ((clk_ctrl1 == 1'b0) && (clk_ctrl0 == 1'b0)) ? `max_bunsyuu_reg_val_2Hz
                               : (((clk_ctrl1 == 1'b0) && (clk_ctrl0 == 1'b1)) ? `max_bunsyuu_reg_val_200Hz
                                 : (((clk_ctrl1 == 1'b1) && (clk_ctrl0 == 1'b0)) ? `max_bunsyuu_reg_val_1kHz
                                   : `max_bunsyuu_reg_val_2Hz));


  always @(posedge clk or negedge reset) begin
     if (reset == 1'b0) begin
	 bunsyuu_reg <= 24'b000000000000000000000000;  // bunsyuu_reg = 0
	 clock_reg   <=                         1'b0;  //     clk_reg = 0
     end else begin
	 if (bunsyuu_reg >= max_bunsyuu_reg_val) begin
	     bunsyuu_reg <= 24'b000000000000000000000000;  // bunsyuu_reg = 0

             clock_reg   <= ~clock_reg;              // clk_reg = not clk_reg
	 end else begin
             bunsyuu_reg <= bunsyuu_reg + 1;
	 end
     end
  end  // always @(posedge clk or negedge reset) begin
  // ʬ
  //
  //

  // Wires
  wire  [11:0]       rom_addr;  // ROM
  wire  [31:0]       rom_data;  // ROM

  wire  [12:0]       ram_addr;  // RAM
  wire       ram_write_enable;  // RAM
  wire  [31:0] ram_write_data;  // RAM
  wire  [31:0]  ram_read_data;  // RAM
  wire  [12:0]  key_ram_addr;   // RAM
  wire  [31:0] key_ram_wdata;   // RAM
  wire          key_ram_wen;    // RAM

  wire  [31:0]   alu_a, alu_b;  // ALU
  wire   [3:0] 	     alu_ctrl;  // ALU
  wire  [31:0]          alu_y;  // ALU
  wire   [1:0]       alu_comp;  // ALU

  wire   [31:0]       pc_next;  // PC
  wire   [31:0]            pc;  // PC

  wire   [4:0]  reg_read_idx1;  // 쥸ե
  wire   [4:0]  reg_read_idx2;  // 쥸ե
  wire   [4:0]  reg_write_idx;  // 쥸ե
  wire  [31:0] reg_write_data;  // 쥸ե
  wire  [31:0] reg_read_data1;  // 쥸ե
  wire  [31:0] reg_read_data2;  // 쥸ե

  wire   [31:0]         inc_a;  // PC 󥯥ȥ⥸塼
  wire   [31:0]         inc_y;  // PC 󥯥ȥ⥸塼

  wire   [31:0]         add_a;  // PC û⥸塼
  wire   [31:0]         add_b;  // PC û⥸塼
  wire   [31:0]         add_y;  // PC û⥸塼

  wire  [31:0]   alu_b_sel1_d0;  // alu_b ϩ1⥸塼
  wire  [31:0]   alu_b_sel1_d1;  // alu_b ϩ1⥸塼
  wire            alu_b_sel1_s;  // alu_b ϩ1⥸塼
  wire  [31:0]    alu_b_sel1_y;  // alu_b ϩ1⥸塼

  wire  [31:0]   alu_b_sel2_d0;  // alu_b ϩ2⥸塼
  wire  [31:0]   alu_b_sel2_d1;  // alu_b ϩ2⥸塼
  wire            alu_b_sel2_s;  // alu_b ϩ2⥸塼
  wire  [31:0]    alu_b_sel2_y;  // alu_b ϩ2⥸塼

  wire  [31:0]   alu_ram_sel_d0; // alu_ram ϩ⥸塼
  wire  [31:0]   alu_ram_sel_d1; // alu_ram ϩ⥸塼
  wire            alu_ram_sel_s; // alu_ram ϩ2⥸塼
  wire  [31:0]    alu_ram_sel_y; // alu_ram ϩ⥸塼

  wire  [31:0]      link_sel_d0;    // link ϩ⥸塼
  wire  [31:0]      link_sel_d1;    // link ϩ⥸塼
  wire               link_sel_s;    // link ϩ⥸塼
  wire  [31:0]       link_sel_y;    // link ϩ⥸塼

  wire  [4:0]   reg_widx_sel1_d0;  // reg_write_idx ϩ1⥸塼
  wire  [4:0]   reg_widx_sel1_d1;  // reg_write_idx ϩ1⥸塼
  wire           reg_widx_sel1_s;  // reg_write_idx ϩ1⥸塼
  wire  [4:0]    reg_widx_sel1_y;  // reg_write_idx ϩ1⥸塼

  wire  [4:0]     reg_widx_sel2_d0;  // reg_write_idx ϩ2⥸塼
  wire  [4:0]     reg_widx_sel2_d1;  // reg_write_idx ϩ2⥸塼
  wire             reg_widx_sel2_s;  // reg_write_idx ϩ2⥸塼
  wire  [4:0]      reg_widx_sel2_y;  // reg_write_idx ϩ2⥸塼

  wire  [31:0]       pc_sel_d0;  // pc ϩ⥸塼
  wire  [31:0]       pc_sel_d1;  // pc ϩ⥸塼
  wire                pc_sel_s;  // pc ϩ⥸塼
  wire  [31:0]        pc_sel_y;  // pc ϩ⥸塼

//
//ɲ߷ 2 Υҥ(1)jp_sel ѥ磻
//
// wire  [31:0]     jp_sel_d0;  // jp ϩ⥸塼 ǡ 1
// wire  [31:0]     jp_sel_d1;  // jp ϩ⥸塼 ǡ 2
// wire              jp_sel_s;  // jp ϩ⥸塼 쥯ȿ
// wire  [31:0]      jp_sel_y;  // jp ϩ⥸塼 

//
//ɲ߷ 5 ҥ(1)jpr_sel ѥ磻
//

  wire   [31:0]    instruction;  // ᥤϩ
  wire              alu_b_sel1;  // ᥤϩ
  wire              alu_b_sel2;  // ᥤϩ
  wire   [2:0]          alu_op;  // ᥤϩ
  wire   [2:0]  is_branch_ctrl;  // ᥤϩ
  wire        reg_write_enable;  // ᥤϩ
  wire                      jp;  // ᥤϩ
  wire                    link;  // ᥤϩ
  wire                     jpr;  // ᥤϩ
  wire                  shiftv;  // ᥤϩ
  wire             do_sign_ext;  // ᥤϩ

  wire   [5:0]            func;  // ALU ϩ⥸塼

  wire   [31:0]           sh_a;  // shifter(2-bit left)
  wire   [31:0]           sh_y;  // shifter(2-bit left)

  wire   [31:0]         sh_j_a;  // shifter(2-bit left) (jump)
  wire   [31:0]         sh_j_y;  // shifter(2-bit left) (jump)

  wire   [15:0]            a16;  // ĥϩ
  wire   [31:0]            y32;  // ĥϩ

  wire    [2:0]   is_branch_d0;  // is_branchϩ
  wire    [1:0]   is_branch_d1;  // is_branchϩ
  wire             is_branch_y;  // is_branchϩ

  wire  [31:0]   alu_a_sel1_d0;  // alu_a ϩ1⥸塼
  wire  [31:0]   alu_a_sel1_d1;  // alu_a ϩ1⥸塼
  wire            alu_a_sel1_s;  // alu_a ϩ1⥸塼
  wire  [31:0]    alu_a_sel1_y;  // alu_a ϩ1⥸塼

  //
  //
  // Body   
  //

  //                  +----+
  //             clk->|    |
  //  rom_addr[11:0]->|    |->rom_data[31:0]
  //                  +----+
  //
  //ߥ졼
  //DE2, quartus, ư¸ȥ
  //rom8x1024_sim  rom8x1024a(rom_addr, rom_data);
  //
  //ư¸
  //ߥ졼ȥ
  rom8x1024_DE2  rom8x1024a(clock_rom, rom_addr, rom_data);

  //                        +----+
  //                   clk->|    |
  //        ram_addr[12:0]->|    |
  //      ram_write_enable->|    |->ram_read_data[31:0]
  //  ram_write_data[31:0]->|    |
  //    key_ram_addr[12:0]->|    |
  //   key_ram_wdata[31:0]->|    |
  //           key_ram_wen->|    |
  //                        +----+
  //
  //ߥ졼
  //DE2, quartus, ư¸ȥ
  //ram8x2048_sim  ram8x2048a(clk, ram_addr, ram_write_enable,
  //                          ram_write_data, ram_read_data,
  //                          key_ram_addr, key_ram_wdata, key_ram_wen);
  //
  //ư¸
  //ߥ졼ȥ
  ram8x2048_DE2  ram8x2048a(clock_ram, ram_addr, ram_write_enable,
                            ram_write_data, ram_read_data,
                            key_ram_addr, key_ram_wdata, key_ram_wen);

// ¸ 9 ΥҥȡʣˡALU ⥸塼μβ˴ؤ뵭Ҥѹ
//    clock, reset ɲá軻̤ݻ쥸 hi  lo 
  // alu
  //                  +----+
  //     alu_a[31:0]->|    |
  //     alu_b[31:0]->|    |->alu_y[31:0]
  //   alu_ctrl[3:0]->|    |->alu_comp[1:0]
  //                  +----+
  alu  alua(alu_a, alu_b, alu_ctrl, alu_y, alu_comp);

  // pc
  //                +----+
  //         clock->|    |
  //         reset->|    |
  // pc_next[31:0]->|    |->pc[31:0]
  //                +----+
  pc  pca (clock, reset, pc_next, pc); 

  // registers
  //                       +----+
  //                clock->|    |
  //                reset->|    |
  //   reg_read_idx1[4:0]->|    |
  //   reg_read_idx2[4:0]->|    |->reg_read_data1[31:0]
  //   reg_write_idx[4:0]->|    |->reg_read_data2[31:0]
  //     reg_write_enable->|    |
  // reg_write_data[31:0]->|    |
  //                       +----+
  registers register_file(clock, reset,
		  reg_read_idx1, reg_read_idx2,
		  reg_write_idx, reg_write_enable, reg_write_data,
		  reg_read_data1, reg_read_data2);

  // plus4
  //              +----+
  // inc_a[31:0]->|    |->inc_y[31:0]
  //              +----+
  plus4 pc_plus4 (inc_a, inc_y);   


  // adder32
  //              +----+
  // add_a[31:0]->|    |
  // add_b[31:0]->|    |->add_y[31:0]
  //              +----+
  adder32 pc_add(add_a, add_b, add_y);

  // mux32_32_32
  //           +----+
  // d0[31:0]->|    |
  // d1[31:0]->|    |->y[31:0]
  //        s->|    |
  //           +----+
  mux32_32_32  alu_b_sel_1(alu_b_sel1_d0, alu_b_sel1_d1, alu_b_sel1_s, alu_b_sel1_y);
  mux32_32_32  alu_b_sel_2(alu_b_sel2_d0, alu_b_sel2_d1, alu_b_sel2_s, alu_b_sel2_y);
  mux32_32_32  alu_ram_sel(alu_ram_sel_d0, alu_ram_sel_d1, alu_ram_sel_s, alu_ram_sel_y);
  mux32_32_32  link_sel(link_sel_d0, link_sel_d1, link_sel_s, link_sel_y);
  mux32_32_32  alu_a_sel_1(alu_a_sel1_d0, alu_a_sel1_d1, alu_a_sel1_s, alu_a_sel1_y);

//
//ɲ߷ 5 ҥ(2)32-bit, 32-bit , 32-bit ϤΥ쥯β
//

//
//ɲ߷ 2 Υҥ(2)32-bit, 32-bit , 32-bit ϤΥ쥯β
//
// mux32_32_32  jp_sel(jp_sel_d0, jp_sel_d1, jp_sel_s, jp_sel_y);

  mux32_32_32  pc_sel(pc_sel_d0, pc_sel_d1, pc_sel_s, pc_sel_y);

  // mux5_5_5
  //           +----+
  //  d0[4:0]->|    |
  //  d1[4:0]->|    |->y[4:0]
  //        s->|    |
  //           +----+
  mux5_5_5  reg_widx_sel1(reg_widx_sel1_d0, reg_widx_sel1_d1,
                            reg_widx_sel1_s, reg_widx_sel1_y);
  mux5_5_5  reg_widx_sel2(reg_widx_sel2_d0, reg_widx_sel2_d1,
                        reg_widx_sel2_s, reg_widx_sel2_y);

  // main_ctrl
  //                     +----+
  //  instruction[31:0]->|    |->ram_write_enable
  //                     |    |->alu_b_sel1_s
  //                     |    |->alu_b_sel2_s
  //                     |    |->alu_op[2:0]
  //                     |    |->is_branch_ctrl[2:0]
  //                     |    |->reg_write_enable
  //                     |    |->alu_ram_sel_s
  //                     |    |->reg_widx_sel1_s
  //                     |    |->jp
  //                     |    |->link
  //                     |    |->jpr
  //                     |    |->shiftv
  //                     |    |->do_sign_ext
  //                     +----+
  main_ctrl main_ctrla(instruction, ram_write_enable, alu_b_sel1, alu_b_sel2, alu_op,
                       is_branch_ctrl, reg_write_enable, 
                       alu_ram_sel_s, reg_widx_sel1_s, jp, link, jpr,
                       shiftv, do_sign_ext);

  // alu_ctrler
  //              +----+
  // alu_op[2:0]->|    |
  //   func[5:0]->|    |->alu_ctrl[3:0]
  //              +----+
  alu_ctrler alu_ctrlera(alu_op, func, alu_ctrl);

  // shifter32_32_l2
  //             +----+
  // sh_a[31:0]->|    |->sh_y[31:0]
  //             +----+
  shifter32_32_l2 shifter32_32_l2a(sh_a, sh_y);
  shifter32_32_l2 shifter32_32_l2b(sh_j_a, sh_j_y);

  // signext16_32
  //              +----+
  // do_sign_ext->|    |
  //   a16[15:0]->|    |->y32[31:0]
  //              +----+
  signext16_32 signext16_32a(do_sign_ext, a16, y32);

  // is_branch
  //           +----+
  //  d0[2:0]->|    |
  //  d1[1:0]->|    |->y
  //           +----+
  is_branch is_brancha(is_branch_d0, is_branch_d1, is_branch_y);

  assign rom_addr = (pc[31:0] > 32'h00400fff) ? 12'hfff : pc[11:0];
  // assign rom_addr = pc[11:0];
  assign ram_addr = alu_y[12:0];
  assign ram_write_data = reg_read_data2;
  assign key_ram_addr   = i_ram_addr;
  assign key_ram_wdata  = i_ram_wdata;
  assign key_ram_wen    = i_ram_wen;
  assign alu_a = alu_a_sel1_y;
  assign alu_b = alu_b_sel2_y;

//
//ɲ߷ 5 ҥ(3)jpr_sel ν jpr_sel_y  pc_next ؤ³
//

//
//ɲ߷ 2 Υҥ(3)jp_sel ν jp_sel_y  pc_next ؤ³
//
// assign pc_next = jp_sel_y;
  assign pc_next = pc_sel_y;

  assign reg_read_idx1 = instruction[25:21];
  assign reg_read_idx2 = instruction[20:16];
  assign reg_write_idx = reg_widx_sel2_y;
  assign reg_write_data = link_sel_y;
  assign inc_a = pc;
  assign add_a = inc_y;
  assign add_b = sh_y;
  assign alu_b_sel1_d0 = reg_read_data2;
  assign alu_b_sel1_d1 = y32;
  assign alu_b_sel1_s = alu_b_sel1;
  assign alu_b_sel2_d0 = alu_b_sel1_y;
  assign alu_b_sel2_d1 = 0;
  assign alu_b_sel2_s = alu_b_sel2;
  assign alu_ram_sel_d0 = alu_y;
  assign alu_ram_sel_d1 = ram_read_data;
  assign reg_widx_sel1_d0 = instruction[20:16];
  assign reg_widx_sel1_d1 = instruction[15:11];
  assign reg_widx_sel2_d0 = reg_widx_sel1_y;
  assign reg_widx_sel2_d1 = 5'b11111;  // RA  idx, 31
  assign reg_widx_sel2_s = link;
  assign link_sel_d0 = alu_ram_sel_y;
  assign link_sel_d1 = inc_y;
  assign link_sel_s = link;
  assign pc_sel_d0 = inc_y;
  assign pc_sel_d1 = add_y;
  assign pc_sel_s = is_branch_y;

//
//ɲ߷ 2 ҥ(4)jp_sel  jp_sel_d0, jp_sel_d1, jp_sel_s ³
//
// assign jp_sel_d0 = pc_sel_y;
// assign jp_sel_d1 = sh_j_y;
// assign jp_sel_s = jp;

//
//ɲ߷ 5 ҥ(4)jpr_sel  jpr_sel_d0, jpr_sel_d1, jpr_sel_s ³
//

  assign instruction = rom_data;
  assign func = y32[5:0];
  assign sh_a = y32;
  assign sh_j_a = {6'b000000, instruction[25:0]};
  assign a16 = instruction[15:0];
  assign is_branch_d0 = is_branch_ctrl;
  assign is_branch_d1 = alu_comp;

  assign alu_a_sel1_d0 = reg_read_data1;
  assign alu_a_sel1_d1 = {27'b000000000000000000000000000, y32[10:6]};
  assign alu_a_sel1_s = shiftv;

  assign o_pc        = pc[11:0];
  assign o_clock     = clock;
  assign o_alu_y     = alu_y;
  assign o_alu_a     = alu_a;
  assign o_alu_b     = alu_b;
  assign o_alu_ctrl  = alu_ctrl;
  assign o_alu_comp  = alu_comp;
  assign o_reg_data1 = reg_read_data1;
  assign o_reg_idx1  = reg_read_idx1;
  assign o_reg_data2 = reg_read_data2;
  assign o_reg_idx2  = reg_read_idx2;
  assign o_reg_wdata = reg_write_data;
  assign o_reg_widx  = reg_write_idx;
  assign o_reg_wen   = reg_write_enable;
  assign o_ram_addr  = ram_addr;
  assign o_ram_rdata = ram_read_data;
  assign o_ram_wdata = ram_write_data;
  assign o_ram_wen   = ram_write_enable;
   
endmodule
   
